"""
colorviews module
"""

__all__ = ["Color", "AlphaColor", "scale"]
__version__ = "0.1-alpha1"
__author__ = "Julin Shaji"

from colorviews.colors import Color, AlphaColor
from colorviews.utils import scale
