_.from_name  # unused method (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/colors.py:102)
_.copy  # unused method (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/colors.py:205)
_.from_name  # unused method (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/colors.py:251)
_.copy  # unused method (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/colors.py:367)
colors  # unused import (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/views.py:23)
_.replace  # unused method (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/views.py:117)
_.replace  # unused method (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/views.py:166)
_.replace  # unused method (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/views.py:272)
_.replace  # unused method (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/views.py:325)
_.replace  # unused method (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/views.py:434)
_.replace  # unused method (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/views.py:487)
